'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var EditorJS = _interopDefault(require('@editorjs/editorjs'));
var Header = _interopDefault(require('@editorjs/header'));
var List = _interopDefault(require('@editorjs/list'));
var InlineCode = _interopDefault(require('@editorjs/inline-code'));
var SimpleImage = _interopDefault(require('@editorjs/simple-image'));
var Embed = _interopDefault(require('@editorjs/embed'));
var Quote = _interopDefault(require('@editorjs/quote'));
var Marker = _interopDefault(require('@editorjs/marker'));
var Code = _interopDefault(require('@editorjs/code'));
var Link = _interopDefault(require('@editorjs/link'));
var Delimiter = _interopDefault(require('@editorjs/delimiter'));
var Raw = _interopDefault(require('@editorjs/raw'));
var Table = _interopDefault(require('@editorjs/table'));
var Warning = _interopDefault(require('@editorjs/warning'));
var Paragraph = _interopDefault(require('@editorjs/paragraph'));
var Checklist = _interopDefault(require('@editorjs/checklist'));

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

var commonTools = {
  header: Header,
  list: List,
  image: SimpleImage,
  inlineCode: InlineCode,
  embed: Embed,
  quote: Quote,
  marker: Marker,
  code: Code,
  link: Link,
  delimiter: Delimiter,
  raw: Raw,
  table: Table,
  warning: Warning,
  paragraph: Paragraph,
  checklist: Checklist
};

var Editor =
/*#__PURE__*/
function (_Component) {
  _inherits(Editor, _Component);

  function Editor(props) {
    var _this;

    _classCallCheck(this, Editor);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Editor).call(this, props));

    _defineProperty(_assertThisInitialized(_this), "_initEditor", function () {
      var _this$props = _this.props,
          holderId = _this$props.holderId,
          autofocus = _this$props.autofocus,
          data = _this$props.data;
      _this.editor = new EditorJS({
        holderId: holderId,
        autofocus: autofocus,
        data: data,
        tools: _this._tools,
        onChange: _this._handleChange,
        onReady: _this._handleReady
      });
    });

    _defineProperty(_assertThisInitialized(_this), "_destroyEditor", function () {
      if (!_this.editor) return;

      _this.editor.destroy();

      _this.editor = null;
    });

    _defineProperty(_assertThisInitialized(_this), "_initTools", function () {
      var _this$props2 = _this.props,
          customTools = _this$props2.customTools,
          excludeDefaultTools = _this$props2.excludeDefaultTools;

      var toolsList = _objectSpread({}, commonTools, customTools);

      if (excludeDefaultTools.length !== 0) {
        return Object.keys(toolsList).filter(function (tool) {
          return !excludeDefaultTools.includes(tool);
        }).reduce(function (acc, curr) {
          return _objectSpread({}, acc, _defineProperty({}, curr, toolsList[curr]));
        }, {});
      }

      return toolsList;
    });

    _defineProperty(_assertThisInitialized(_this), "_handleChange",
    /*#__PURE__*/
    _asyncToGenerator(
    /*#__PURE__*/
    regeneratorRuntime.mark(function _callee() {
      var data;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return _this.editor.save();

            case 2:
              data = _context.sent;

              _this._onChange(data);

            case 4:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    })));

    _defineProperty(_assertThisInitialized(_this), "_handleReady", function () {
      _this._onReady();
    });

    _this._tools = _this._initTools(props.tools, props.excludeTools);
    _this._onChange = props.onChange;
    _this._onReady = props.onReady;
    _this._el = React__default.createRef();
    return _this;
  }

  _createClass(Editor, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      this._initEditor();
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      this._destroyEditor();
    }
  }, {
    key: "render",
    value: function render() {
      var holderId = this.props.holderId;
      return React__default.createElement('div', {
        id: holderId,
        ref: this._el
      });
    }
  }]);

  return Editor;
}(React.Component);

_defineProperty(Editor, "defaultProps", {
  holderId: 'editorjs-holder',
  customTools: {},
  excludeDefaultTools: [],
  onChange: function onChange() {},
  onReady: function onReady() {},
  data: {},
  autofocus: true
});

module.exports = Editor;
